@@ -4,25 +4,18 @@
 
 #include "acsyscap.h"
 
-#define ACVERSION "0.7.27"
+#define ACVERSION "0.8.0"
 
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 #include <climits>
 
-// keep this consistently, for experiments with other STL implementations
-#define MYSTD std
-//#define MYSTD ustl
-//#include <ustl.h>
-
-#define MYMAP MYSTD::map
-
 #ifdef HAVE_MEMORY_SPTR
 #include <memory>
-#define SMARTPTR_SPACE MYSTD
+#define SMARTPTR_SPACE std
 #elif defined HAVE_TR1_MEMORY
 #include <tr1/memory>
-#define SMARTPTR_SPACE MYSTD::tr1
+#define SMARTPTR_SPACE std::tr1
 #elif defined HAVE_BOOST_SMARTPTR
 #include <boost/smart_ptr.hpp>
 #define SMARTPTR_SPACE boost
@@ -30,21 +23,13 @@
 #error Unable to find smart pointer implementation, install Boost or recent compiler with STL containing TR1 components. Set BOOSTDIR in Makefile if needed.
 #endif
 
-#ifndef HAVE_ZLIB
-#warning Zlib or its development files are not available. Install them (e.g. zlib1g-dev) and run "make clean". Gzip format support disabled.
-#endif
-
-#ifndef HAVE_LIBBZ2
-#warning LibBz2 or its development files are not available. Install them (e.g. libbz2-dev) and run "make clean". Bzip2 format support disabled.
-#endif
-
 // make off_t be a 64 bit type
 // added in Makefile... #define _FILE_OFFSET_BITS 64
 
 #define SHARED_PTR SMARTPTR_SPACE::shared_ptr
 #define INTRUSIVE_PTR SMARTPTR_SPACE::intrusive_ptr
 #define WEAK_PTR SMARTPTR_SPACE::weak_ptr
-#define SCOPED_PTR MYSTD::auto_ptr
+#define SCOPED_PTR std::auto_ptr
 
 #ifdef NO_EXCEPTIONS
 #define MYTRY
