@@ -21,25 +21,46 @@
 #define ASSERT(x)
 #define LOG(x)
 #define LOGSTART(x)
+#define LOGSTARTs(x)
 #define LOGSTART2(x,y)
 #define LOGSTART2s(x,y)
 #define DBGQLOG(x)
+#define dbgprint(x)
+inline void dump_proc_status(){}; // strip away
 
 #else
 
+#include <fstream>
+#include <iostream>
+
 #define LOGLVL(n, x) if(acfg::debug&n){ __logobj.GetFmter() << x; __logobj.Write(__FILE__,__LINE__); }
 #define LOG(x) LOGLVL(LOG_DEBUG, x)
 
 #define LOGSTART(x) t_logger __logobj(x, this);
 #define LOGSTARTs(x) t_logger __logobj(x, NULL);
-#define LOGSTART2(x, y) t_logger __logobj(x, this); LOGLVL(LOG_DEBUG, y)
-#define LOGSTART2s(x, y) t_logger __logobj(x, NULL); LOGLVL(LOG_DEBUG, y)
+#define LOGSTART2(x, y) t_logger __logobj(x, this); LOGLVL(LOG_DEBUG, y /* << "@" __FILE__ ":" << __LINE__  */ )
+#define LOGSTART2s(x, y) t_logger __logobj(x, NULL); LOGLVL(LOG_DEBUG, y /*<< "@" __FILE__ ":" << __LINE__ */ )
+
+#define dbgprint(x) std::cerr << x << std::endl;
 
 #define ldbg(x) LOG(x)
 
 #define dbgline ldbg("mark")
 #define DBGQLOG(x) {aclog::err(tSS()<< x);}
 
+
+inline void dump_proc_status()
+{
+	using namespace std;
+	ifstream sf("/proc/self/status");
+	while (sf)
+	{
+		string s;
+		getline(sf, s);
+		cerr << s << endl;
+	}
+};
+
 #endif
 
 #endif // __DEBUG_H__
