@@ -9,17 +9,17 @@
 #include "acbuf.h"
 #include "fileio.h"
 #include "cleaner.h"
+#include "filelocks.h"
 
 #include <errno.h>
 #include <algorithm>
 
-using namespace MYSTD;
+using namespace std;
 
 #define MAXTEMPDELAY acfg::maxtempdelay // 27
 mstring sReplDir("_altStore" SZPATHSEP);
 
 static tFiGlobMap mapItems;
-lockable mapLck;
 
 header const & fileitem::GetHeaderUnlocked()
 {
@@ -250,10 +250,25 @@ bool fileitem::SetupClean(bool bForce)
 			return false;
 		m_status=FIST_INITED;
 	}
-	cmstring sPathAbs(CACHE_BASE+m_sPathRel);
-	if(::truncate(sPathAbs.c_str(), 0) || ::truncate((sPathAbs+".head").c_str(), 0))
-		return false;
-
+	cmstring sPathAbs(SABSPATH(m_sPathRel));
+	cmstring sPathHead(sPathAbs+".head");
+	// header allowed to be lost in process...
+//	if(unlink(sPathHead.c_str()))
+//		::ignore_value(::truncate(sPathHead.c_str(), 0));
+	::ignore_value(::truncate(sPathAbs.c_str(), 0));
+	Cstat stf(sPathAbs);
+	if(stf && stf.st_size>0)
+		return false; // didn't work. Permissions? Anyhow, too dangerous to act on this now
+	header h;
+	h.LoadFromFile(sPathHead);
+	h.del(header::CONTENT_LENGTH);
+	h.del(header::CONTENT_TYPE);
+	h.del(header::LAST_MODIFIED);
+	h.del(header::XFORWARDEDFOR);
+	h.del(header::CONTENT_RANGE);
+	h.StoreToFile(sPathHead);
+//	if(0==stat(sPathHead.c_str(), &stf) && stf.st_size >0)
+//		return false; // that's weird too, header still exists with real size
 	m_head.clear();
 	m_nSizeSeen=m_nSizeChecked=0;
 
@@ -280,33 +295,31 @@ fileitem::FiStatus fileitem::WaitForFinish(int *httpCode)
 
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
-	errnoFmter f;
+	tErrnoFmter f;
 	aclog::err(tSS() << sFile <<
 			" storage error [" << msg << "], last errno: " << f);
 }
 
 #ifndef MINIBUILD
 
-/*
-#ifdef DEBUG
-#define SET_FRONTLINE(ret, x) tSS __fbuf; __fbuf << "HTTP/1.1 " << x << " @@ " << __FILE__ ":" \
-	<< __LINE__ << " /" << __FUNCTION__; ret=__fbuf;
-#else
-#endif
-*/
-
-#define SET_FRONTLINE(ret, x) ret="HTTP/1.1 "; ret+=x;
-#define SETERROR(x) { m_bAllowStoreData=false; SET_FRONTLINE(m_head.frontLine, x); \
-	m_head.set(header::XORIG, h.h[header::XORIG]); \
-    m_status=FIST_DLERROR; m_nTimeDlDone=GetTime(); _LogWithErrno(x, m_sPathRel); }
-#define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
-#define BOUNCE(x) { SETERROR(x); return false; }
-
 bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *pNextData,
 		bool bForcedRestart, bool &bDoCleanRetry)
 {
 	LOGSTART("fileitem::DownloadStartedStoreHeader");
 
+	auto SETERROR = [&](LPCSTR x) {
+		m_bAllowStoreData=false;
+		m_head.frontLine=mstring("HTTP/1.1 ")+x;
+		m_head.set(header::XORIG, h.h[header::XORIG]);
+	    m_status=FIST_DLERROR; m_nTimeDlDone=GetTime();
+		_LogWithErrno(x, m_sPathRel);
+	};
+
+	auto withError = [&](LPCSTR x) {
+		SETERROR(x);
+		return false;
+	};
+
 	setLockGuard;
 
 	USRDBG( "Download started, storeHeader for " << m_sPathRel << ", current status: " << m_status);
@@ -329,12 +342,32 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 	// optional optimization: hints for the filesystem resp. kernel
 	off_t hint_start(0), hint_length(0);
 	
-	// status will change, most likely... ie. BOUNCE action
+	// status will change, most likely... ie. return withError action
 	notifyAll();
 
 	cmstring sPathAbs(CACHE_BASE+m_sPathRel);
 	string sHeadPath=sPathAbs + ".head";
 
+	auto withErrorAndKillFile = [&](LPCSTR x)
+	{
+		SETERROR(x);
+		if(m_filefd>=0)
+		{
+#if _POSIX_SYNCHRONIZED_IO > 0
+			fsync(m_filefd);
+#endif
+			forceclose(m_filefd);
+		}
+
+		LOG("Deleting " << sPathAbs);
+		::unlink(sPathAbs.c_str());
+		::unlink(sHeadPath.c_str());
+
+		m_status=FIST_DLERROR;
+		m_nTimeDlDone=GetTime();
+		return false;
+	};
+
 	int serverStatus = h.getStatus();
 #if 0
 #warning FIXME
@@ -359,12 +392,12 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 				 * Most likely the remote file was modified after the download started.
 				 */
 				//USRDBG( "state: " << m_status << ", m_nsc: " << m_nSizeChecked);
-				BOUNCE("500 Failed to resume remote download");
+				return withError("500 Failed to resume remote download");
 			}
 			if(h.h[header::CONTENT_LENGTH] && atoofft(h.h[header::CONTENT_LENGTH])
 					!= atoofft(h.h[header::CONTENT_LENGTH], -2))
 			{
-				BOUNCE("500 Failed to resume remote download, bad length");
+				return withError("500 Failed to resume remote download, bad length");
 			}
 			m_head.set(header::XORIG, h.h[header::XORIG]);
 		}
@@ -382,7 +415,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		if(m_nSizeSeen<=0 && m_nRangeLimit<0)
 		{
 			// wtf? Cannot have requested partial content
-			BOUNCE("500 Unexpected Partial Response");
+			return withError("500 Unexpected Partial Response");
 		}
 		/*
 		 * Range: bytes=453291-
@@ -392,7 +425,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		 */
 		const char *p=h.h[header::CONTENT_RANGE];
 		if(!p)
-			BOUNCE("500 Missing Content-Range in Partial Response");
+			return withError("500 Missing Content-Range in Partial Response");
 		off_t myfrom, myto, mylen;
 		int n=sscanf(p, "bytes " OFF_T_FMT "-" OFF_T_FMT "/" OFF_T_FMT, &myfrom, &myto, &mylen);
 		if(n<=0)
@@ -406,7 +439,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 				|| myfrom<0 || mylen<0
 		)
 		{
-			BOUNCE("500 Server reports unexpected range");
+			return withError("500 Server reports unexpected range");
 		}
 
 		m_nSizeChecked=myfrom;
@@ -463,7 +496,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		// -> kill cached file ASAP
 			m_bAllowStoreData=false;
 			m_head.copy(h, header::XORIG);
-			SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
+			return withErrorAndKillFile("503 Server disagrees on file size, cleaning up");
 		}
 		break;
 	default:
@@ -475,7 +508,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 			// got an error from the replacement mirror? cannot handle it properly
 			// because some job might already have started returning the data
 			USRDBG( "Cannot restart, HTTP code: " << serverStatus);
-			BOUNCE(h.getCodeMessage());
+			return withError(h.getCodeMessage());
 		}
 
 		m_bAllowStoreData=false;
@@ -503,6 +536,10 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		struct stat stbuf;
 				
 		mkbasedir(sPathAbs);
+
+		// this makes sure not to truncate file while it's mmaped
+		auto tempLock = filelocks::Acquire(sPathAbs);
+
 		m_filefd=open(sPathAbs.c_str(), flags, acfg::fileperms);
 		ldbg("file opened?! returned: " << m_filefd);
 		
@@ -515,17 +552,17 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 				if(FileCopy(sPathAbs, temp) && 0==unlink(sPathAbs.c_str()) )
 				{
 					if(0!=rename(temp.c_str(), sPathAbs.c_str()))
-						BOUNCE("503 Cannot rename files");
+						return withError("503 Cannot rename files");
 					
 					// be sure about that
 					if(0!=stat(sPathAbs.c_str(), &stbuf) || stbuf.st_size!=m_nSizeSeen)
-						BOUNCE("503 Cannot copy file parts, filesystem full?");
+						return withError("503 Cannot copy file parts, filesystem full?");
 					
 					m_filefd=open(sPathAbs.c_str(), flags, acfg::fileperms);
 					ldbg("file opened after copying around: ");
 				}
 				else
-					BOUNCE((tSS()<<"503 Cannot store or remove files in "
+					return withError((tSS()<<"503 Cannot store or remove files in "
 							<< GetDirPart(sPathAbs)).c_str());
 			}
 			else
@@ -538,16 +575,16 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		
 		if (m_filefd<0)
 		{
-			errnoFmter efmt("503 Cache storage error - ");
+			tErrnoFmter efmt("503 Cache storage error - ");
 #ifdef DEBUG
-			BOUNCE((efmt+sPathAbs).c_str());
+			return withError((efmt+sPathAbs).c_str());
 #else
-			BOUNCE(efmt.c_str());
+			return withError(efmt.c_str());
 #endif
 		}
 		
 		if(0!=fstat(m_filefd, &stbuf) || !S_ISREG(stbuf.st_mode))
-			SETERRORKILLFILE("503 Not a regular file");
+			return withErrorAndKillFile("503 Not a regular file");
 		
 		// crop, but only if the new size is smaller. MUST NEVER become larger (would fill with zeros)
 		if(m_nSizeChecked <= m_nSizeSeen)
@@ -559,10 +596,10 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 #endif
       }
 			else
-				SETERRORKILLFILE("503 Cannot change file size");
+				return withErrorAndKillFile("503 Cannot change file size");
 		}
 		else if(m_nSizeChecked>m_nSizeSeen) // should never happen and caught by the checks above
-			SETERRORKILLFILE("503 Internal error on size checking");
+			return withErrorAndKillFile("503 Internal error on size checking");
 		// else... nothing to fix since the expectation==reality
 
 		falloc_helper(m_filefd, hint_start, hint_length);
@@ -573,40 +610,23 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		posix_fadvise(m_filefd, hint_start, hint_length, POSIX_FADV_SEQUENTIAL);
 #endif
 */
-		
 		ldbg("Storing header as "+sHeadPath);
 		int count=m_head.StoreToFile(sHeadPath);
 
 		if(count<0)
-			SETERRORKILLFILE( (-count!=ENOSPC ? "503 Cache storage error" : "503 OUT OF DISK SPACE"));
+			return withErrorAndKillFile( (-count!=ENOSPC
+					? "503 Cache storage error" : "503 OUT OF DISK SPACE"));
 			
 		// double-check the sane state
 		if(0!=fstat(m_filefd, &stbuf) || stbuf.st_size!=m_nSizeChecked)
-			SETERRORKILLFILE("503 Inconsistent file state");
+			return withErrorAndKillFile("503 Inconsistent file state");
 			
 		if(m_nSizeChecked!=lseek(m_filefd, m_nSizeChecked, SEEK_SET))
-			SETERRORKILLFILE("503 IO error, positioning");
+			return withErrorAndKillFile("503 IO error, positioning");
 	}
 	
 	m_status=FIST_DLGOTHEAD;
 	return true;
-
-	kill_file:
-	if(m_filefd>=0)
-	{
-#if _POSIX_SYNCHRONIZED_IO > 0
-		fsync(m_filefd);
-#endif
-		forceclose(m_filefd);
-	}
-
-	LOG("Deleting " << sPathAbs);
-	unlink(sPathAbs.c_str());
-	unlink(sHeadPath.c_str());
-	
-	m_status=FIST_DLERROR;
-	m_nTimeDlDone=GetTime();
-	return false;
 }
 
 bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
@@ -661,7 +681,7 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 				{
 					if(EINTR==errno || EAGAIN==errno)
 						continue;
-					errnoFmter efmt("HTTP/1.1 503 ");
+					tErrnoFmter efmt("HTTP/1.1 503 ");
 					m_head.frontLine = efmt;
 					m_status=FIST_DLERROR;
 					// message will be set by the caller
@@ -693,16 +713,17 @@ inline void fileItemMgmt::Unreg()
 {
 	LOGSTART("fileItemMgmt::Unreg");
 
-	if(!m_ptr)
+	if(!m_ptr) // unregistered before?
 		return;
 
-	lockguard managementLock(mapLck);
+	lockguard managementLock(mapItems);
 
 	// invalid or not globally registered?
 	if(m_ptr->m_globRef == mapItems.end())
 		return;
 
-	lockguard fitemLock(*m_ptr);
+	auto local_ptr(m_ptr); // might disappear
+	lockguard fitemLock(*local_ptr);
 
 	if ( -- m_ptr->usercount <= 0)
 	{
@@ -729,18 +750,21 @@ inline void fileItemMgmt::Unreg()
 		LOG("*this is last entry, deleting dl/fi mapping");
 		mapItems.erase(m_ptr->m_globRef);
 		m_ptr->m_globRef = mapItems.end();
+
+		// make sure it's not double-unregistered accidentally!
+		m_ptr.reset();
 	}
 }
 
 
-fileItemMgmt fileItemMgmt::GetRegisteredFileItem(cmstring &sPathUnescaped, bool bConsiderAltStore)
+bool fileItemMgmt::PrepageRegisteredFileItemWithStorage(cmstring &sPathUnescaped, bool bConsiderAltStore)
 {
-	LOGSTART2s("fileitem::GetFileItem", sPathUnescaped);
+	LOGSTART2("fileitem::GetFileItem", sPathUnescaped);
 
 	MYTRY
 	{
 		mstring sPathRel(fileitem_with_storage::NormalizePath(sPathUnescaped));
-		lockguard lockGlobalMap(mapLck);
+		lockguard lockGlobalMap(mapItems);
 		tFiGlobMap::iterator it=mapItems.find(sPathRel);
 		if(it!=mapItems.end())
 		{
@@ -758,9 +782,8 @@ fileItemMgmt fileItemMgmt::GetRegisteredFileItem(cmstring &sPathUnescaped, bool
 						{
 							it->second->usercount++;
 							LOG("Sharing an existing REPLACEMENT file item");
-							fileItemMgmt ret;
-							ret.m_ptr = it->second;
-							return ret;
+							m_ptr = it->second;
+							return true;
 						}
 					}
 					// ok, then create a modded name version in the replacement directory
@@ -772,16 +795,14 @@ fileItemMgmt fileItemMgmt::GetRegisteredFileItem(cmstring &sPathUnescaped, bool
 					p->usercount=1;
 					tFileItemPtr sp(p);
 					p->m_globRef = mapItems.insert(make_pair(sPathRel, sp));
-					fileItemMgmt ret;
-					ret.m_ptr = sp;
-					return ret;
+					m_ptr = sp;
+					return true;
 				}
 			}
 			LOG("Sharing existing file item");
 			it->second->usercount++;
-			fileItemMgmt ret;
-			ret.m_ptr = it->second;
-			return ret;
+			m_ptr = it->second;
+			return true;
 		}
 		LOG("Registering the NEW file item...");
 		fileitem_with_storage *p = new fileitem_with_storage(sPathRel);
@@ -789,34 +810,43 @@ fileItemMgmt fileItemMgmt::GetRegisteredFileItem(cmstring &sPathUnescaped, bool
 		tFileItemPtr sp(p);
 		p->m_globRef = mapItems.insert(make_pair(sPathRel, sp));
 		//lockGlobalMap.unLock();
-		fileItemMgmt ret;
-		ret.m_ptr = sp;
-		return ret;
+		m_ptr = sp;
+		return true;
 	}
-	MYCATCH(MYSTD::bad_alloc&)
+	MYCATCH(std::bad_alloc&)
 	{
 	}
-	return fileItemMgmt();
+	return false;
 }
 
 // make the fileitem globally accessible
-fileItemMgmt fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
+bool fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
 {
-	if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
-		return fileItemMgmt();
+	LOGSTART2("fileitem::RegisterFileItem", spCustomFileItem->m_sPathRel);
 
-	lockguard lockGlobalMap(mapLck);
+	if (!spCustomFileItem || spCustomFileItem->m_sPathRel.empty())
+		return false;
+
+	Unreg();
+
+	lockguard lockGlobalMap(mapItems);
 
 	if(ContHas(mapItems, spCustomFileItem->m_sPathRel))
-		return fileItemMgmt(); // conflict, another agent is already active
+		return false; // conflict, another agent is already active
 
 	spCustomFileItem->m_globRef = mapItems.insert(make_pair(spCustomFileItem->m_sPathRel,
 			spCustomFileItem));
 
 	spCustomFileItem->usercount=1;
-	fileItemMgmt ret;
-	ret.m_ptr = spCustomFileItem;
-	return ret;
+	m_ptr = spCustomFileItem;
+	return true;
+}
+
+void fileItemMgmt::RegisterFileitemLocalOnly(fileitem* replacement)
+{
+	LOGSTART2("fileItemMgmt::ReplaceWithLocal", replacement);
+	Unreg();
+	m_ptr.reset(replacement);
 }
 
 
@@ -826,7 +856,7 @@ fileItemMgmt fileItemMgmt::RegisterFileItem(tFileItemPtr spCustomFileItem)
 time_t fileItemMgmt::BackgroundCleanup()
 {
 	LOGSTART2s("fileItemMgmt::BackgroundCleanup", GetTime());
-	lockguard lockGlobalMap(mapLck);
+	lockguard lockGlobalMap(mapItems);
 	tFiGlobMap::iterator it, here;
 
 	time_t now=GetTime();
@@ -844,7 +874,7 @@ time_t fileItemMgmt::BackgroundCleanup()
 		// find and ignore (but remember) the candidate(s) for the next cycle
 		if (here->second->m_nTimeDlStarted > expBefore)
 		{
-			oldestGet = MYSTD::min(here->second->m_nTimeDlStarted, oldestGet);
+			oldestGet = std::min(time_t(here->second->m_nTimeDlStarted), oldestGet);
 			continue;
 		}
 
@@ -866,7 +896,7 @@ time_t fileItemMgmt::BackgroundCleanup()
 	ldbg(oldestGet);
 
 	// preserving a few seconds to catch more of them in the subsequent run
-	return MYSTD::max(oldestGet + MAXTEMPDELAY, GetTime()+8);
+	return std::max(oldestGet + MAXTEMPDELAY, GetTime()+8);
 }
 
 ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos, size_t count)
@@ -887,25 +917,25 @@ void fileItemMgmt::dump_status()
 {
 	tSS fmt;
 	aclog::err("File descriptor table:\n");
-	for(tFiGlobMap::iterator it=mapItems.begin(); it!=mapItems.end(); ++it)
+	for(const auto& item : mapItems)
 	{
 		fmt.clear();
-		fmt << "FREF: " << it->first << " [" << it->second->usercount << "]:\n";
-		if(! it->second)
+		fmt << "FREF: " << item.first << " [" << item.second->usercount << "]:\n";
+		if(! item.second)
 		{
 			fmt << "\tBAD REF!\n";
 			continue;
 		}
 		else
 		{
-			fmt << "\t" << it->second->m_sPathRel
-					<< "\n\tDlRefCount: " << it->second->m_nDlRefsCount
-					<< "\n\tState: " << it->second->m_status
-					<< "\n\tFilePos: " << it->second->m_nIncommingCount << " , "
-					<< it->second->m_nRangeLimit << " , "
-					<< it->second->m_nSizeChecked << " , "
-					<< it->second->m_nSizeSeen
-					<< "\n\tGotAt: " << it->second->m_nTimeDlStarted << "\n\n";
+			fmt << "\t" << item.second->m_sPathRel
+					<< "\n\tDlRefCount: " << item.second->m_nDlRefsCount
+					<< "\n\tState: " << item.second->m_status
+					<< "\n\tFilePos: " << item.second->m_nIncommingCount << " , "
+					<< item.second->m_nRangeLimit << " , "
+					<< item.second->m_nSizeChecked << " , "
+					<< item.second->m_nSizeSeen
+					<< "\n\tGotAt: " << item.second->m_nTimeDlStarted << "\n\n";
 		}
 		aclog::err(fmt.c_str(), NULL);
 	}
@@ -917,29 +947,8 @@ fileitem_with_storage::~fileitem_with_storage()
 	if(startsWith(m_sPathRel, sReplDir))
 	{
 		::unlink(SZABSPATH(m_sPathRel));
-		unlink((SABSPATH(m_sPathRel)+".head").c_str());
+		::unlink((SABSPATH(m_sPathRel)+".head").c_str());
 	}
 }
 
-fileItemMgmt::fileItemMgmt(const fileItemMgmt &src)
-{
-	fileItemMgmt *x = const_cast<fileItemMgmt *>(&src);
-	this->m_ptr = x->m_ptr;
-	x->m_ptr.reset();
-}
-
-fileItemMgmt& fileItemMgmt::operator=(const fileItemMgmt &src)
-{
-	fileItemMgmt *x = const_cast<fileItemMgmt *>(&src);
-	this->m_ptr = x->m_ptr;
-	x->m_ptr.reset();
-	return *this;
-}
-
-void fileItemMgmt::ReplaceWithLocal(fileitem* replacement)
-{
-	Unreg();
-	m_ptr.reset(replacement);
-}
-
 #endif // MINIBUILD
