@@ -7,13 +7,25 @@
 #include <sys/mman.h>
 #include <errno.h>
 #include <limits>
+#include <signal.h>
 
 #include "md5.h"
 #include "sha1.h"
 #include "csmapping.h"
+#include "aclogger.h"
+#include "filelocks.h"
+#include "lockable.h"
+#include "debug.h"
 
 #include <iostream>
 
+#ifdef HAVE_SSL
+#include <openssl/sha.h>
+#include <openssl/md5.h>
+#endif
+
+//#define SHRINKTEST
+
 // must be something sensible, ratio impacts stack size by inverse power of 2
 #define BUFSIZEMIN 4095 // makes one page on i386 and should be enough for typical index files
 #define BUFSIZEMAX 16*4096
@@ -23,37 +35,68 @@
 #undef HAVE_LIBBZ2
 #undef HAVE_ZLIB
 #undef HAVE_LZMA
+#else
+
+#ifndef HAVE_ZLIB
+#warning Zlib or its development files are not available. Install them (e.g. zlib1g-dev) and run "make distclean". Gzip format support disabled.
 #endif
 
-using namespace MYSTD;
+#ifndef HAVE_LIBBZ2
+#warning LibBz2 or its development files are not available. Install them (e.g. libbz2-dev) and run "make distclean". Bzip2 format support disabled.
+#endif
 
+#endif
+
+using namespace std;
+
+// to make sure not to deal with incomplete operations from a signal handler
+class tMmapEntry
+{
+public:
+	std::atomic_bool valid;
+	pthread_t threadref;
+	string path;
+	tMmapEntry() : threadref() {
+		valid.store(false);
+	}
+};
+
+// this weird kludge is only needed in order to access this data from POSIX signal handlers
+// which needs to be both, reliable and lock-free
+tMmapEntry g_mmapMemory[10];
+lockable g_mmapMemoryLock;
 
 class IDecompressor
 {
 public:
-	bool inited, eof;
-	IDecompressor() : inited(false), eof(false) {};
+	bool eof=false;
+	mstring *psError = NULL;
 	virtual ~IDecompressor() {};
 	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf) =0;
+	virtual bool Init() =0;
 };
 
 #ifdef HAVE_LIBBZ2
 #include <bzlib.h>
 class tBzDec : public IDecompressor
 {
-	bz_stream strm;
+	bz_stream strm = bz_stream();
 public:
-	tBzDec()
+	bool Init() override
 	{
-		::memset(&strm, 0, sizeof(strm));
-		inited = (BZ_OK == BZ2_bzDecompressInit(&strm, 1, EXTREME_MEMORY_SAVING));
+		if (BZ_OK == BZ2_bzDecompressInit(&strm, 1, EXTREME_MEMORY_SAVING))
+			return true;
+		// crap, no proper sanity checks in BZ2_bzerror
+		if(psError)
+			psError->assign("BZIP2 initialization error");
+		return false;
+
 	}
 	~tBzDec()
 	{
 		BZ2_bzDecompressEnd(&strm);
-		inited = false;
 	}
-	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf) override
 	{
 		strm.next_in = szInBuf + nBufPos;
 		strm.avail_in = nBufSize - nBufPos;
@@ -64,37 +107,42 @@ public:
 		if (ret == BZ_STREAM_END || ret == BZ_OK)
 		{
 			nBufPos = nBufSize - strm.avail_in;
-			UINT nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			uint nGotBytes = UncompBuf.freecapa() - strm.avail_out;
 			UncompBuf.got(nGotBytes);
 			eof = ret == BZ_STREAM_END;
 			return true;
 		}
 		// or corrupted data?
 		eof = true;
+		// eeeeks bz2, no robust error getter, no prepared error message :-(
+		if(psError)
+			*psError = mstring("BZIP2 error ")+ltos(ret);
 		return false;
 	}
 };
-#else
-#define tBzDec IDecompressor
+static const uint8_t bz2Magic[] =
+{ 'B', 'Z', 'h' };
 #endif
 
 #ifdef HAVE_ZLIB
 #include <zlib.h>
 class tGzDec : public IDecompressor
 {
-	z_stream strm;
+	z_stream strm = z_stream();
 public:
-	tGzDec()
+	bool Init() override
 	{
-		::memset(&strm, 0, sizeof(strm));
-		inited = (Z_OK == inflateInit2(&strm, 47));
+		if (Z_OK == inflateInit2(&strm, 47))
+			return true;
+		if (psError)
+			psError->assign("ZLIB initialization error");
+		return false;
 	}
 	~tGzDec()
 	{
 		deflateEnd(&strm);
-		inited = false;
 	}
-	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf) override
 	{
 		strm.next_in = (uint8_t*) szInBuf + nBufPos;
 		strm.avail_in = nBufSize - nBufPos;
@@ -105,19 +153,20 @@ public:
 		if (ret == Z_STREAM_END || ret == Z_OK)
 		{
 			nBufPos = nBufSize - strm.avail_in;
-			UINT nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			uint nGotBytes = UncompBuf.freecapa() - strm.avail_out;
 			UncompBuf.got(nGotBytes);
 			eof = ret == Z_STREAM_END;
 			return true;
 		}
 		// or corrupted data?
 		eof = true;
+		if(psError)
+			*psError = mstring("ZLIB error: ") + (strm.msg ? mstring(strm.msg) : ltos(ret));
 		return false;
 	}
 };
-
-#else
-#define tGzDec IDecompressor
+static const uint8_t gzMagic[] =
+{ 0x1f, 0x8b, 0x8 };
 #endif
 
 #ifdef HAVE_LZMA
@@ -125,25 +174,29 @@ public:
 
 class tXzDec : public IDecompressor
 {
-	lzma_stream strm;
+	lzma_stream strm = lzma_stream();
+	bool lzmaFormat;
 public:
-	tXzDec(bool lzmaFormat=false)
+	tXzDec(bool bLzma) : lzmaFormat(bLzma) {};
+
+	bool Init() override
 	{
-		::memset(&strm, 0, sizeof(strm));
-		if(lzmaFormat)
-			inited = (LZMA_OK == lzma_alone_decoder(&strm,
-					EXTREME_MEMORY_SAVING ? 32000000 : MAX_VAL(uint64_t)));
-		else
-			inited = (LZMA_OK == lzma_stream_decoder(&strm,
+		auto x = (lzmaFormat ? lzma_alone_decoder(&strm,
+					EXTREME_MEMORY_SAVING ? 32000000 : MAX_VAL(uint64_t))
+				: lzma_stream_decoder(&strm,
 				EXTREME_MEMORY_SAVING ? 32000000 : MAX_VAL(uint64_t),
 						LZMA_TELL_UNSUPPORTED_CHECK | LZMA_CONCATENATED));
+		if(LZMA_OK == x)
+			return true;
+		if(psError)
+			psError->assign("LZMA initialization error");
+		return false;
 	}
 	~tXzDec()
 	{
 		lzma_end(&strm);
-		inited = false;
 	}
-	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf) override
 	{
 		strm.next_in = (uint8_t*) szInBuf + nBufPos;
 		strm.avail_in = nBufSize - nBufPos;
@@ -154,26 +207,24 @@ public:
 		if (ret == LZMA_STREAM_END || ret == LZMA_OK)
 		{
 			nBufPos = nBufSize - strm.avail_in;
-			UINT nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			uint nGotBytes = UncompBuf.freecapa() - strm.avail_out;
 			UncompBuf.got(nGotBytes);
 			eof = ret == LZMA_STREAM_END;
 			return true;
 		}
 		// or corrupted data?
 		eof = true;
+		if(psError)
+			*psError = mstring("LZMA error ")+ltos(ret);
 		return false;
 	}
 };
-#else
-class tXzDec : public IDecompressor
-{
-public:
-	tXzDec() {}
-	tXzDec(bool) {}
-};
+static const uint8_t xzMagic[] =
+{ 0xfd, '7', 'z', 'X', 'Z', 0x0 },
+lzmaMagic[] = {0x5d, 0, 0, 0x80};
 #endif
 
-filereader::filereader() 
+filereader::filereader()
 :
 	m_bError(false),
 	m_bEof(false),
@@ -186,27 +237,38 @@ filereader::filereader()
 {
 };
 
-static const uint8_t gzMagic[] =
-{ 0x1f, 0x8b, 0x8 }, bz2Magic[] =
-{ 'B', 'Z', 'h' }, xzMagic[] =
-{ 0xfd, '7', 'z', 'X', 'Z', 0x0 },
-lzmaMagic[] = {0x5d, 0, 0, 0x80};
-
-bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
+bool filereader::OpenFile(const string & sFilename, bool bNoMagic, uint nFakeTrailingNewlines)
 {
 	Close(); // reset to clean state
-	
+	m_nEofLines=nFakeTrailingNewlines;
+
+	// this makes sure not to truncate file while it's mmaped
+	m_mmapLock = filelocks::Acquire(sFilename);
+	// namedmutex mmapMx(g_noTruncateLocks, sFilename);
+	// lockguard guardWriteMx(mmapMx);
+
 	m_fd = open(sFilename.c_str(), O_RDONLY);
+#ifdef SHRINKTEST
+	m_fd = open(sFilename.c_str(), O_RDWR);
+#warning destruction mode!!!
+#endif
 
 	if (m_fd < 0)
+	{
+		m_sErrorString=tErrnoFmter();
 		return false;
+	}
 
 	if (bNoMagic)
 		m_Dec.reset();
+#ifdef HAVE_LIBBZ2
 	else if (endsWithSzAr(sFilename, ".bz2"))
 		m_Dec.reset(new tBzDec);
+#endif
+#ifdef HAVE_ZLIB
 	else if (endsWithSzAr(sFilename, ".gz"))
 		m_Dec.reset(new tGzDec);
+#endif
 #ifdef HAVE_LZMA
 	else if(endsWithSzAr(sFilename, ".xz"))
 		m_Dec.reset(new tXzDec(false));
@@ -215,17 +277,22 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 #endif
 	else // unknown... ok, probe it
 	{
-		filereader fh;
-		if (fh.OpenFile(sFilename, true) && fh.GetSize() >= 10)
+		m_UncompBuf.setsize(10);
+		if(m_UncompBuf.sysread(m_fd) >= 10)
 		{
-			if (0 == memcmp(gzMagic, fh.GetBuffer(), _countof(gzMagic)))
+			if(false) {}
+#ifdef HAVE_ZLIB
+			else if (0 == memcmp(gzMagic, m_UncompBuf.rptr(), _countof(gzMagic)))
 				m_Dec.reset(new tGzDec);
-			else if (0 == memcmp(bz2Magic, fh.GetBuffer(), _countof(bz2Magic)))
+#endif
+#ifdef HAVE_LIBBZ2
+			else if (0 == memcmp(bz2Magic, m_UncompBuf.rptr(), _countof(bz2Magic)))
 				m_Dec.reset(new tBzDec);
+#endif
 #ifdef HAVE_LZMA
-			else if (0 == memcmp(xzMagic, fh.GetBuffer(), _countof(xzMagic)))
-				m_Dec.reset(new tXzDec);
-			else if (0 == memcmp(lzmaMagic, fh.GetBuffer(), _countof(lzmaMagic)))
+			else if (0 == memcmp(xzMagic, m_UncompBuf.rptr(), _countof(xzMagic)))
+				m_Dec.reset(new tXzDec(false));
+			else if (0 == memcmp(lzmaMagic, m_UncompBuf.rptr(), _countof(lzmaMagic)))
 				m_Dec.reset(new tXzDec(true));
 #endif
 		}
@@ -233,23 +300,27 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 
 	if (m_Dec.get())
 	{
-		if(!m_Dec->inited)
-		{
-			//aclog::err("Unable to uncompress file, algo not available");
+		m_Dec->psError = &m_sErrorString;
+		if(!m_Dec->Init())
 			return false;
-		}
 		m_UncompBuf.clear();
 		m_UncompBuf.setsize(BUFSIZEMIN);
 	}
 
+	tErrnoFmter fmt;
+
 	struct stat statbuf;
 	if(0!=fstat(m_fd, &statbuf))
+	{
+		m_sErrorString=tErrnoFmter();
 		return false;
+	}
 
 	// LFS on 32bit? That's not good for mmap. Don't risk incorrect behaviour.
 	if(uint64_t(statbuf.st_size) >  MAX_VAL(size_t))
     {
         errno=EFBIG;
+        m_sErrorString=tErrnoFmter();
         return false;
     }
 	
@@ -257,12 +328,16 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	{
 		m_szFileBuf = (char*) mmap(0, statbuf.st_size, PROT_READ, MAP_SHARED, m_fd, 0);
 		if(m_szFileBuf==MAP_FAILED)
-			return false;
+		{
+				m_sErrorString=tErrnoFmter();
+				return false;
+		}
 		m_nBufSize = statbuf.st_size;
 	}
 	else if(m_Dec.get())
 	{
 		// compressed file but empty -> error
+		m_sErrorString="Truncated compressed file";
 		return false;
 	}
 	else
@@ -271,14 +346,38 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 		m_nBufSize = 0;
 	}
 	
+	// ORDER DOES MATTER! No returns anymore before the mmap entry was booked in "the memory"
+
 #ifdef HAVE_MADVISE
 	// if possible, prepare to read that
 	posix_madvise(m_szFileBuf, statbuf.st_size, POSIX_MADV_SEQUENTIAL);
 #endif
 	
+#if 0//def SHRINKTEST
+	if (ftruncate(m_fd, GetSize()/2) < 0)
+	{
+		perror ("ftruncate");
+		::exit(1);
+	}
+#endif
+
 	m_nBufPos=0;
 	m_nCurLine=0;
 	m_bError = m_bEof = false;
+
+	{
+		// try to keep a thread reference in some free slot
+		lockguard g(g_mmapMemoryLock);
+		for(auto &x : g_mmapMemory)
+		{
+			if(!x.valid.load())
+			{
+				x.path=sFilename;
+				x.threadref=pthread_self();
+				x.valid.store(true);
+			}
+		}
+	}
 	return true;
 }
 
@@ -291,7 +390,7 @@ bool filereader::CheckGoodState(bool bErrorsConsiderFatal, cmstring *reportFileP
 	cerr << "Error opening file";
 	if (reportFilePath)
 		cerr << " " << *reportFilePath;
-	cerr << ", terminating." << endl;
+	cerr << " (" << m_sErrorString << "), terminating." << endl;
 	exit(EXIT_FAILURE);
 
 }
@@ -299,9 +398,21 @@ bool filereader::CheckGoodState(bool bErrorsConsiderFatal, cmstring *reportFileP
 void filereader::Close()
 {
 	m_nCurLine=0;
-	
+	m_mmapLock.reset();
+
 	if (m_szFileBuf != MAP_FAILED)
 	{
+		{
+			lockguard g(g_mmapMemoryLock);
+			for (auto &x : g_mmapMemory)
+			{
+				if (x.valid.load()
+				&& pthread_equal(pthread_self(), x.threadref)
+				&& m_mmapLock.get() && x.path == m_mmapLock->path)
+					x.valid = false;
+			}
+		}
+
 		munmap(m_szFileBuf, m_nBufSize);
 		m_szFileBuf = (char*) MAP_FAILED;
 	}
@@ -312,12 +423,66 @@ void filereader::Close()
 	m_nBufSize=0;
 
 	m_bError = m_bEof = true; // will be cleared in Open()
+	m_sErrorString=cmstring("Not initialized");
 }
 
 filereader::~filereader() {
 	Close();
 }
 
+std::atomic_int g_nThreadsToKill;
+
+void handle_sigbus()
+{
+#if 1
+	for (auto &x : g_mmapMemory)
+	{
+		if (!x.valid.load())
+			continue;
+		aclog::err(string("FATAL ERROR: probably IO error occurred, probably while reading the file ")
+			+x.path+" . Please check your system logs for related errors.");
+	}
+
+#else
+	// attempt to pass the signal around threads, stopping the one troublemaker
+	std::cerr << "BUS ARRIVED" << std::endl;
+
+	bool metoo = false;
+
+	if (g_nThreadsToKill.load())
+		goto suicid;
+
+	g_degraded.store(true);
+
+	// this must run lock-free
+	for (auto &x : g_mmapMemory)
+	{
+		if (!x.valid.load())
+			continue;
+		aclog::err(
+				string("FATAL ERROR: probably IO error occurred, probably while reading the file ")
+						+ x.path + " . Please check your system logs for related errors.");
+
+		if (pthread_equal(pthread_self(), x.threadref))
+			metoo = true;
+		else
+		{
+			g_nThreadsToKill.fetch_add(1);
+			pthread_kill(x.threadref, SIGBUS);
+		}
+	}
+
+	if (!metoo)
+		return;
+
+	suicid: aclog::err("FATAL ERROR: SIGBUS, probably caused by an IO error. "
+			"Please check your system logs for related errors.");
+
+	g_nThreadsToKill.fetch_add(-1);
+	pthread_exit(0);
+#endif
+}
+
 bool filereader::GetOneLine(string & sOut, bool bForceUncompress) {
 	
 	sOut.clear();
@@ -341,13 +506,14 @@ bool filereader::GetOneLine(string & sOut, bool bForceUncompress) {
 		{
 			m_UncompBuf.move(); // make free as much as possible
 
-			// must uncompress but the buffer is full. Resize if, it not possible, fail only then.
+			// must uncompress but the buffer is full. Resize it, if not possible, fail only then.
 			if(bForceUncompress && 0==m_UncompBuf.freecapa())
 			{
 				if (m_UncompBuf.totalcapa() >= BUFSIZEMAX
 						|| !m_UncompBuf.setsize(m_UncompBuf.totalcapa() * 2))
 				{
 					m_bError = true;
+					m_sErrorString=mstring("Failed to allocate decompression buffer memory");
 					return false;
 				}
 			}
@@ -374,7 +540,7 @@ bool filereader::GetOneLine(string & sOut, bool bForceUncompress) {
 	//const char *newline=mempbrk(rbuf, "\r\n", nRest);
   //const char *crptr=(const char*) memchr(rbuf, '\r', nRest);
   //const char *lfptr=(const char*) memchr(rbuf, '\n', nRest);
-  //const char *newline = (crptr&&lfptr) ? MYSTD::min(crptr,lfptr) : MYSTD::max(crptr,lfptr);
+  //const char *newline = (crptr&&lfptr) ? std::min(crptr,lfptr) : std::max(crptr,lfptr);
 	const char *newline=0;
 	for(const char *x=rbuf; x<rbuf+nRest; ++x)
 	{
@@ -422,30 +588,48 @@ bool filereader::GetOneLine(string & sOut, bool bForceUncompress) {
 }
 
 #ifndef MINIBUILD
+
+#ifdef HAVE_SSL
+class csumSHA1 : public csumBase, public SHA_CTX
+{
+public:
+	csumSHA1() { SHA1_Init(this); }
+	void add(const char *data, size_t size) override { SHA1_Update(this, (const void*) data, size); }
+	void finish(uint8_t* ret) override { SHA1_Final(ret, this); }
+};
+class csumMD5 : public csumBase, public MD5_CTX
+{
+public:
+	csumMD5() { MD5_Init(this); }
+	void add(const char *data, size_t size) override { MD5_Update(this, (const void*) data, size); }
+	void finish(uint8_t* ret) override { MD5_Final(ret, this); }
+};
+#else
 class csumSHA1 : public csumBase, public SHA_INFO
 {
 public:
 	csumSHA1() { sha_init(this); }
-	void add(const char *data, size_t size) { sha_update(this, (SHA_BYTE*) data, size); }
-	void finish(uint8_t* ret) { sha_final(ret, this); }
+	void add(const char *data, size_t size) override { sha_update(this, (SHA_BYTE*) data, size); }
+	void finish(uint8_t* ret) override { sha_final(ret, this); }
 };
 class csumMD5 : public csumBase, public md5_state_s
 {
 public:
 	csumMD5() { md5_init(this); }
-	void add(const char *data, size_t size) { md5_append(this, (md5_byte_t*) data, size); }
-	void finish(uint8_t* ret) { md5_finish(this, ret); }
+	void add(const char *data, size_t size) override { md5_append(this, (md5_byte_t*) data, size); }
+	void finish(uint8_t* ret) override { md5_finish(this, ret); }
 };
+#endif
 
-auto_ptr<csumBase> csumBase::GetChecker(CSTYPES type)
+std::unique_ptr<csumBase> csumBase::GetChecker(CSTYPES type)
 {
 	switch(type)
 	{
 	case CSTYPE_MD5:
-		return auto_ptr<csumBase>(new csumMD5);
+		return std::unique_ptr<csumBase>(new csumMD5);
 	case CSTYPE_SHA1:
 	default: // for now
-		return auto_ptr<csumBase>(new csumSHA1);
+		return std::unique_ptr<csumBase>(new csumSHA1);
 	}
 }
 
@@ -458,9 +642,8 @@ bool filereader::GetChecksum(const mstring & sFileName, int csType, uint8_t out[
 }
 
 bool filereader::GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE *fDump)
-//bool filereader::GetSha1Sum(uint8_t out[], off_t &scannedSize, FILE *fDump)
 {
-	auto_ptr<csumBase> summer = csumBase::GetChecker(CSTYPES(csType));
+	unique_ptr<csumBase> summer(csumBase::GetChecker(CSTYPES(csType)));
 	scannedSize=0;
 	
 	if(!m_Dec.get())
@@ -482,7 +665,7 @@ bool filereader::GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE
 				return false;
 			}
 
-			UINT nPlainSize=m_UncompBuf.size();
+			uint nPlainSize=m_UncompBuf.size();
 			summer->add(m_UncompBuf.rptr(), nPlainSize);
 			if(fDump)
 				fwrite(m_UncompBuf.rptr(), sizeof(char), nPlainSize, fDump);
@@ -508,7 +691,7 @@ void check_algos()
 {
 	const char testvec[]="abc";
 	uint8_t out[20];
-	auto_ptr<csumBase> ap = csumBase::GetChecker(CSTYPE_SHA1);
+	unique_ptr<csumBase> ap(csumBase::GetChecker(CSTYPE_SHA1));
 	ap->add(testvec, sizeof(testvec)-1);
 	ap->finish(out);
 	if(!CsEqual("a9993e364706816aba3e25717850c26c9cd0d89d", out, 20))
