@@ -6,7 +6,7 @@
 #include <cstdio>
 #include <stdexcept>
 #include <limits>
-using namespace MYSTD;
+using namespace std;
 
 #include "conn.h"
 #include "acfg.h"
@@ -32,6 +32,19 @@ mstring sHttp11("HTTP/1.1");
 #define SPECIAL_FD -42
 inline bool IsValidFD(int fd) { return fd>=0 || SPECIAL_FD == fd; }
 
+tTraceData traceData;
+void acfg::dump_trace()
+{
+	aclog::err("Paths with uncertain content types:");
+	lockguard g(traceData);
+	for (const auto& s : traceData)
+		aclog::err(s);
+}
+tTraceData& tTraceData::getInstance()
+{
+	return traceData;
+}
+
 
 /*
  * Unlike the regular store-and-forward file item handler, this ones does not store anything to
@@ -46,28 +59,29 @@ protected:
 	size_t m_nConsumable, m_nConsumed;
 
 public:
-	tPassThroughFitem(MYSTD::string s) :
+	tPassThroughFitem(std::string s) :
 	m_pData(NULL), m_nConsumable(0), m_nConsumed(0)
 	{
 		m_sPathRel = s;
 		m_bAllowStoreData=false;
 		m_nSizeChecked = m_nSizeSeen = 0;
 	};
-	virtual FiStatus Setup(bool)
+	virtual FiStatus Setup(bool) override
 	{
 		m_nSizeChecked = m_nSizeSeen = 0;
 		return m_status = FIST_INITED;
 	}
-	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
+	virtual int GetFileFd() override
+			{ return SPECIAL_FD; }; // something, don't care for now
 	virtual bool DownloadStartedStoreHeader(const header & h, const char *,
-			bool, bool&)
+			bool, bool&) override
 	{
 		setLockGuard;
 		m_head=h;
 		m_status=FIST_DLGOTHEAD;
 		return true;
 	}
-	virtual bool StoreFileData(const char *data, unsigned int size)
+	virtual bool StoreFileData(const char *data, unsigned int size) override
 	{
 		setLockGuard;
 
@@ -107,7 +121,7 @@ public:
 		}
 		return true;
 	}
-	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow)
+	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow) override
 	{
 		setLockGuard;
 
@@ -122,7 +136,7 @@ public:
 		if(!m_nSizeChecked)
 			return 0;
 
-		ssize_t r = write(out_fd, m_pData, min(nMax2SendNow, m_nConsumable));
+		auto r = write(out_fd, m_pData, min(nMax2SendNow, m_nConsumable));
 		if (r < 0 && (errno == EAGAIN || errno == EINTR)) // harmless
 			r = 0;
 		if(r<0)
@@ -146,7 +160,9 @@ public:
 class tGeneratedFitemBase : public fileitem
 {
 public:
-	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
+	virtual int GetFileFd() override
+	{ return SPECIAL_FD; }; // something, don't care for now
+
 	tSS m_data;
 
 	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg) : m_data(256)
@@ -156,13 +172,14 @@ public:
 		m_head.type = header::ANSWER;
 		m_head.frontLine = "HTTP/1.1 ";
 		m_head.frontLine += (szFrontLineMsg ? szFrontLineMsg : "500 Internal Failure");
-		m_head.set(header::CONTENT_TYPE, _SZ2PS("text/html") );
+		m_head.set(header::CONTENT_TYPE, WITHLEN("text/html") );
 	}
 	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow)
+	override
 	{
 		if (m_status > FIST_COMPLETE || out_fd<0)
 			return -1;
-		int r = m_data.syswrite(out_fd, nMax2SendNow);
+		auto r = m_data.syswrite(out_fd, nMax2SendNow);
 		if(r>0) nSendPos+=r;
 		return r;
 	}
@@ -173,8 +190,10 @@ public:
 		m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
 	}
 	// never used to store data
-	bool DownloadStartedStoreHeader(const header &, const char *, bool, bool&) {return false;};
-	bool StoreFileData(const char *, unsigned int) {return false;};
+	bool DownloadStartedStoreHeader(const header &, const char *, bool, bool&)
+	override
+	{return false;};
+	bool StoreFileData(const char *, unsigned int) override {return false;};
 	header & HeadRef() { return m_head; }
 };
 
@@ -192,7 +211,7 @@ ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 		return -1;
 	while(count>0)
 	{
-		ssize_t readcount=read(in_fd, buf, count>sizeof(buf)?sizeof(buf):count);
+		auto readcount=read(in_fd, buf, min(count, sizeof(buf)));
 		if(readcount<=0)
 		{
 			if(errno==EINTR || errno==EAGAIN)
@@ -205,17 +224,15 @@ ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 		totalcnt+=readcount;
 		count-=readcount;
 		
-		ssize_t nPos(0);
-		while(nPos<readcount)
+		for(decltype(readcount) nPos(0);nPos<readcount;)
 		{
-			ssize_t r=write(out_fd, buf+nPos, readcount-nPos);
+			auto r=write(out_fd, buf+nPos, readcount-nPos);
 			if(r==0) continue; // not nice but needs to deliver it
 			if(r<0)
 			{
 				if(errno==EAGAIN || errno==EINTR)
 					continue;
-				else
-					return r;
+				return r;
 			}
 			nPos+=r;
 		}
@@ -252,9 +269,9 @@ job::~job()
 
 	bool bErr=m_sFileLoc.empty();
 	m_pParentCon->LogDataCounts(
-			( bErr ? (m_pItem ? m_pItem->GetHttpMsg() : miscError ) : m_sFileLoc ),
+			( bErr ? (m_pItem ? m_pItem.get()->GetHttpMsg() : miscError ) : m_sFileLoc ),
 			m_pReqHead->h[header::XFORWARDEDFOR],
-			(m_pItem ? m_pItem->GetTransferCount() : 0),
+			(m_pItem ? m_pItem.get()->GetTransferCount() : 0),
 			m_nAllDataCount, bErr);
 	
 	checkforceclose(m_filefd);
@@ -333,7 +350,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 					seal();
 				}
 			};
-			m_pItem.ReplaceWithLocal(new dirredirect(visPath));
+			m_pItem.RegisterFileitemLocalOnly(new dirredirect(visPath));
 			return;
 		}
 
@@ -347,7 +364,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 			}
 		};
 		listing *p=new listing(visPath);
-		m_pItem.ReplaceWithLocal(p); // assign to smart pointer ASAP, operations might throw
+		m_pItem.RegisterFileitemLocalOnly(p); // assign to smart pointer ASAP, operations might throw
 		tSS & page = p->m_data;
 
 		page << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"
@@ -395,9 +412,10 @@ inline void job::HandleLocalDownload(const string &visPath,
 			for(tStrMap::const_iterator it=sortMap.begin(); it!=sortMap.end(); it++)
 				page << "<tr><td valign=\"top\">" << it->second << "</td></tr>\r\n";
 		}
-		page << "<tr><td colspan=\"4\">";
-		_AddFooter(page);
-		page << "</td></tr></table></body></html>";
+		cmstring& GetFooter();
+		page << "<tr><td colspan=\"4\">"
+		<<GetFooter()
+		<< page << "</td></tr></table></body></html>";
 		p->seal();
 		return;
 	}
@@ -428,7 +446,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 			if(!sMimeType.empty())
 				m_head.set(header::CONTENT_TYPE, sMimeType);
 		};
-		virtual int GetFileFd()
+		virtual int GetFileFd() override
 		{
 			int fd=open(m_sPathRel.c_str(), O_RDONLY);
 		#ifdef HAVE_FADVISE
@@ -439,7 +457,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 			return fd;
 		}
 	};
-	m_pItem.ReplaceWithLocal(new tLocalGetFitem(absPath, stbuf));
+	m_pItem.RegisterFileitemLocalOnly(new tLocalGetFitem(absPath, stbuf));
 }
 
 inline bool job::ParseRange()
@@ -485,11 +503,11 @@ void job::PrepareDownload() {
 #endif
 
     string sReqPath, sPathResidual;
-    tHttpUrl tUrl; // parsed URL
+    tHttpUrl theUrl; // parsed URL
+
+	// resolve to an internal repo location and maybe backends later
+	acfg::tRepoResolvResult repoMapping;
 
-    const acfg::tRepoData * pBackends(NULL); // appropriate backends
-    const string * psVname(NULL); // virtual name for storage pool, if available
-    
     fileitem::FiStatus fistate(fileitem::FIST_FRESH);
     bool bPtMode(false);
     bool bForceFreshnessChecks(false); // force update of the file, i.e. on dynamic index files?
@@ -526,19 +544,19 @@ void job::PrepareDownload() {
 		if (0==sReqPath.compare(0, 11, "apt-cacher/"))
 		sReqPath.erase(11);
 		
-		if(!tUrl.SetHttpUrl(sReqPath, false))
+		if(!theUrl.SetHttpUrl(sReqPath, false))
 		{
-			m_sMaintCmd="/";
+			m_eMaintWorkType=tSpecialRequest::workUSERINFO;
 			return;
 		}
-		LOG("refined path: " << tUrl.sPath);
+		LOG("refined path: " << theUrl.sPath << "\n on host: " << theUrl.sHost);
 
 		char *pEnd(0);
-		UINT nPort = 80;
-		LPCSTR sPort=tUrl.GetPort().c_str();
+		uint nPort = 80;
+		LPCSTR sPort=theUrl.GetPort().c_str();
 		if(!*sPort)
 		{
-			nPort = (UINT) strtoul(sPort, &pEnd, 10);
+			nPort = (uint) strtoul(sPort, &pEnd, 10);
 			if('\0' != *pEnd || pEnd == sPort || nPort > TCP_PORT_MAX || !nPort)
 				goto report_invport;
 		}
@@ -553,25 +571,20 @@ void job::PrepareDownload() {
 		}
 
 		// kill multiple slashes
-		for(tStrPos pos; stmiss != (pos = tUrl.sPath.find("//")); )
-			tUrl.sPath.erase(pos, 1);
+		for(tStrPos pos; stmiss != (pos = theUrl.sPath.find("//")); )
+			theUrl.sPath.erase(pos, 1);
 
-		bPtMode=rechecks::MatchUncacheableRequest(tUrl.ToURI(false));
+		bPtMode=rechecks::MatchUncacheable(theUrl.ToURI(false), rechecks::NOCACHE_REQ);
 
-		LOG("input uri: "<<tUrl.ToURI(false)<<" , dontcache-flag? " << bPtMode);
+		LOG("input uri: "<<theUrl.ToURI(false)<<" , dontcache-flag? " << bPtMode);
 
-		tStrPos nRepNameLen=acfg::reportpage.size();
-		if(nRepNameLen>0)
+		if(!acfg::reportpage.empty() || theUrl.sHost == "style.css")
 		{
-			if(startsWith(tUrl.sHost, acfg::reportpage))
+			m_eMaintWorkType = tSpecialRequest::DispatchMaintWork(sReqPath,
+					m_pReqHead->h[header::AUTHORIZATION]);
+			if(m_eMaintWorkType != tSpecialRequest::workNotSpecial)
 			{
-				m_sMaintCmd=sReqPath;
-				return;
-			}
-			if (tUrl.sHost == "style.css")
-			{
-				LOG("support CSS style file");
-				m_sMaintCmd = "/style.css";
+				m_sFileLoc = sReqPath;
 				return;
 			}
 		}
@@ -579,44 +592,49 @@ void job::PrepareDownload() {
 		using namespace rechecks;
 
 		{
-			tStrMap::const_iterator it = acfg::localdirs.find(tUrl.sHost);
+			tStrMap::const_iterator it = acfg::localdirs.find(theUrl.sHost);
 			if (it != acfg::localdirs.end())
 			{
-				HandleLocalDownload(sReqPath, it->second, tUrl.sPath);
+				HandleLocalDownload(sReqPath, it->second, theUrl.sPath);
 				return;
 			}
 		}
 
 		// entered directory but not defined as local? Then 404 it with hints
-		if(!tUrl.sPath.empty() && endsWithSzAr(tUrl.sPath, "/"))
+		if(!theUrl.sPath.empty() && endsWithSzAr(theUrl.sPath, "/"))
 		{
-			LOG("generic user information page");
-			m_sMaintCmd="/";
+			LOG("generic user information page for " << theUrl.sPath);
+			m_eMaintWorkType=tSpecialRequest::workUSERINFO;
 			return;
 		}
 
-		m_type = GetFiletype(tUrl.sPath);
+		m_type = GetFiletype(theUrl.sPath);
 
 		if ( m_type == FILE_INVALID )
-			goto report_notallowed;
+		{
+			if(!acfg::patrace)
+				goto report_notallowed;
+
+			// ok, collect some information helpful to the user
+			m_type = FILE_VOLATILE;
+			lockguard g(traceData);
+			traceData.insert(theUrl.sPath);
+		}
 		
 		// got something valid, has type now, trace it
 		USRDBG("Processing new job, "<<m_pReqHead->frontLine);
 
-		// resolve to an internal location
-		psVname = acfg::GetRepNameAndPathResidual(tUrl, sPathResidual);
-		
-		if(psVname)
-			m_sFileLoc=*psVname+SZPATHSEP+sPathResidual;
+		acfg::GetRepNameAndPathResidual(theUrl, repoMapping);
+		if(repoMapping.psRepoName && !repoMapping.psRepoName->empty())
+			m_sFileLoc=*repoMapping.psRepoName+SZPATHSEP+repoMapping.sRestPath;
 		else
-			m_sFileLoc=tUrl.sHost+tUrl.sPath;
+			m_sFileLoc=theUrl.sHost+theUrl.sPath;
 
-		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type == FILE_INDEX);
-
-    	m_pItem=fileItemMgmt::GetRegisteredFileItem(m_sFileLoc, bForceFreshnessChecks);
+		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type == FILE_VOLATILE);
+		m_pItem.PrepageRegisteredFileItemWithStorage(m_sFileLoc, bForceFreshnessChecks);
 
 	}
-	MYCATCH(MYSTD::out_of_range&) // better safe...
+	MYCATCH(std::out_of_range&) // better safe...
 	{
     	goto report_invpath;
     }
@@ -627,11 +645,11 @@ void job::PrepareDownload() {
     	goto report_overload;
     }
     
-    fistate = m_pItem->Setup(bForceFreshnessChecks);
+    fistate = m_pItem.get()->Setup(bForceFreshnessChecks);
 	LOG("Got initial file status: " << fistate);
 
 	if (bPtMode && fistate != fileitem::FIST_COMPLETE)
-		fistate = _SwitchToPtItem(m_sFileLoc);
+		fistate = _SwitchToPtItem();
 
 	ParseRange();
 
@@ -644,8 +662,9 @@ void job::PrepareDownload() {
 	if((m_nReqRangeFrom>=0 && m_nReqRangeTo>=0)
 			|| (m_pReqHead->type==header::HEAD && 0!=(m_nReqRangeTo=-1)))
 	{
-		lockguard g(m_pItem.get().get());
-		if(m_pItem->CheckUsableRange_unlocked(m_nReqRangeTo))
+		auto p(m_pItem.get());
+		lockguard g(p.get());
+		if(m_pItem.get()->CheckUsableRange_unlocked(m_nReqRangeTo))
 		{
 			LOG("Got a partial request for incomplete download; range is available");
 			m_bNoDownloadStarted=true;
@@ -668,35 +687,37 @@ void job::PrepareDownload() {
     	}
     	
     	dbgline;
-    	MYTRY
+MYTRY
 		{
-			if(psVname && NULL != (pBackends=acfg::GetBackendVec(*psVname)))
-			{
-				LOG("Backends found, using them with " << sPathResidual
-						<< ", first backend: " <<pBackends->m_backends.front().ToURI(false));
+    		auto bHaveRedirects=(repoMapping.repodata && !repoMapping.repodata->m_backends.empty());
 
-				if(! bPtMode && rechecks::
-						MatchUncacheableTarget(pBackends->m_backends.front().ToURI(false)+sPathResidual))
+    		if (acfg::forcemanaged && !bHaveRedirects)
+						goto report_notallowed;
+
+				if (!bPtMode)
 				{
-					fistate=_SwitchToPtItem(m_sFileLoc);
+					// XXX: this only checks the first found backend server, what about others?
+					auto testUri= bHaveRedirects
+							? repoMapping.repodata->m_backends.front().ToURI(false)
+									+ repoMapping.sRestPath
+							: theUrl.ToURI(false);
+					if (rechecks::MatchUncacheable(testUri, rechecks::NOCACHE_TGT))
+						fistate = _SwitchToPtItem();
 				}
 
-				m_pParentCon->m_pDlClient->AddJob(m_pItem.get(), pBackends, sPathResidual);
-			}
-			else
-			{
-			    if(acfg::forcemanaged)
-			    	goto report_notallowed;
-			    LOG("Passing new job for " << tUrl.ToURI(false) << " to " << m_pParentCon->m_dlerthr);
-
-				if(! bPtMode && rechecks::MatchUncacheableTarget(tUrl.ToURI(false)))
-					fistate=_SwitchToPtItem(m_sFileLoc);
-
-			    m_pParentCon->m_pDlClient->AddJob(m_pItem.get(), tUrl);
-			}
-			ldbg("Download job enqueued for " << m_sFileLoc);
+					if (m_pParentCon->m_pDlClient->AddJob(m_pItem.get(),
+							bHaveRedirects ? nullptr : &theUrl, repoMapping.repodata,
+							bHaveRedirects ? &repoMapping.sRestPath : nullptr))
+				{
+					ldbg("Download job enqueued for " << m_sFileLoc);
+				}
+				else
+				{
+					ldbg("PANIC! Error creating download job for " << m_sFileLoc);
+					goto report_overload;
+				}
 		}
-		MYCATCH(MYSTD::bad_alloc&) // OOM, may this ever happen here?
+		MYCATCH(std::bad_alloc&) // OOM, may this ever happen here?
 		{
 			USRDBG( "Out of memory");
 			goto report_overload;
@@ -735,14 +756,14 @@ report_doubleproxy:
 }
 
 #define THROW_ERROR(x) { if(m_nAllDataCount) return R_DISCON; SetErrorResponse(x); return R_AGAIN; }
-job::eJobResult job::SendData(int confd, int nAllJobCount)
+job::eJobResult job::SendData(int confd)
 {
 	LOGSTART("job::SendData");
 
-	if(!m_sMaintCmd.empty())
+	if(m_eMaintWorkType)
 	{
-		DispatchAndRunMaintTask(m_sMaintCmd, confd, m_pReqHead->h[header::AUTHORIZATION]);
-			return R_DISCON; // just stop and close connection
+		tSpecialRequest::RunMaintWork(m_eMaintWorkType, m_sFileLoc, confd);
+		return R_DISCON; // just stop and close connection
 	}
 	
 	off_t nGoodDataSize(0);
@@ -758,12 +779,12 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 		
 		for(;;)
 		{
-			fistate=m_pItem->GetStatusUnlocked(nGoodDataSize);
+			fistate=m_pItem.get()->GetStatusUnlocked(nGoodDataSize);
 			
 			LOG(fistate);
 			if (fistate > fileitem::FIST_COMPLETE)
 			{
-				const header &h = m_pItem->GetHeaderUnlocked();
+				const header &h = m_pItem.get()->GetHeaderUnlocked();
 				g.unLock(); // item lock must be released in order to replace it!
 				if(m_nAllDataCount)
 					return R_DISCON;
@@ -794,12 +815,12 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				break;
 			
 			dbgline;
-			m_pItem->wait();
+			m_pItem.get()->wait();
 			
 			dbgline;
 		}
 		
-		respHead = m_pItem->GetHeaderUnlocked();
+		respHead = m_pItem.get()->GetHeaderUnlocked();
 
 		if(respHead.h[header::XORIG])
 			m_sOrigUrl=respHead.h[header::XORIG];
@@ -842,7 +863,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 						return R_AGAIN;
 					}
 
-					m_filefd=m_pItem->GetFileFd();
+					m_filefd=m_pItem.get()->GetFileFd();
 					if(!IsValidFD(m_filefd)) THROW_ERROR("503 IO error");
 
 					m_state=m_bChunkMode ? STATE_SEND_CHUNK_HEADER : STATE_SEND_PLAIN_DATA;
@@ -865,7 +886,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 
 					size_t nMax2SendNow=min(nGoodDataSize-m_nSendPos, m_nCurrentRangeLast+1-m_nSendPos);
 					ldbg("~sendfile: on "<< m_nSendPos << " up to : " << nMax2SendNow);
-					int n = m_pItem->SendData(confd, m_filefd, m_nSendPos, nMax2SendNow);
+					int n = m_pItem.get()->SendData(confd, m_filefd, m_nSendPos, nMax2SendNow);
 					ldbg("~sendfile: " << n << " new m_nSendPos: " << m_nSendPos);
 
 					if(n>0)
@@ -885,6 +906,11 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				{
 					m_nChunkRemainingBytes=nGoodDataSize-m_nSendPos;
 					ldbg("STATE_SEND_CHUNK_HEADER for " << m_nChunkRemainingBytes);
+					if(!m_nChunkRemainingBytes && fistate < fileitem::FIST_COMPLETE)
+					{
+						ldbg("No data to send YET, will try later");
+						return R_AGAIN;
+					}
 					m_sendbuf << tSS::hex << m_nChunkRemainingBytes << tSS::dec
 							<< (m_nChunkRemainingBytes ? "\r\n" : "\r\n\r\n");
 
@@ -898,7 +924,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 
 					if(m_nChunkRemainingBytes==0)
 						GOTOENDE; // done
-					int n = m_pItem->SendData(confd, m_filefd, m_nSendPos, m_nChunkRemainingBytes);
+					int n = m_pItem.get()->SendData(confd, m_filefd, m_nSendPos, m_nChunkRemainingBytes);
 					if(n<0)
 						THROW_ERROR("400 Client error");
 					m_nChunkRemainingBytes-=n;
@@ -920,7 +946,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					MYTRY
 					{
 						ldbg("prebuf sending: "<< m_sendbuf.c_str());
-						int r=send(confd, m_sendbuf.rptr(), m_sendbuf.size(),
+						auto r=send(confd, m_sendbuf.rptr(), m_sendbuf.size(),
 								m_backstate == STATE_TODISCON ? 0 : MSG_MORE);
 						if (r<0)
 						{
@@ -1014,7 +1040,7 @@ inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 			m_bChunkMode=true;
 			sb<<"Transfer-Encoding: chunked\r\n";
 		}
-		else if(200==httpstatus) // good data response with known length, can try some optimizations
+		else if(200==httpstatus) // state: good data response with known length, can try some optimizations
 		{
 			LOG("has known content length, optimizing response...");
 
@@ -1025,10 +1051,10 @@ inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 			const char *pLastMo = respHead.h[header::LAST_MODIFIED];
 
 			// consider contents "fresh" for non-volatile data, or when "our" special client is there, or the client simply doesn't care
-			bool bFreshnessForced = (m_type != rechecks::FILE_INDEX
+			bool bDataIsFresh = (m_type != rechecks::FILE_VOLATILE
 				|| m_pReqHead->h[header::ACNGFSMARK] || !pIfmo);
 
-			struct tm tm1=tm(), tm2=tm();
+			auto tm1=tm(), tm2=tm();
 			bool bIfModSeenAndChecked=false;
 			if(pIfmo && header::ParseDate(pIfmo, &tm1) && header::ParseDate(pLastMo, &tm2))
 			{
@@ -1038,31 +1064,31 @@ inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 			}
 
 			// is it fresh? or is this relevant? or is range mode forced?
-			if(  bFreshnessForced || bIfModSeenAndChecked)
+			if(  bDataIsFresh || bIfModSeenAndChecked)
 			{
 				off_t nContLen=atoofft(respHead.h[header::CONTENT_LENGTH]);
 
+				// Client requested with Range* spec?
 				if(m_nReqRangeFrom >=0)
 				{
 					if(m_nReqRangeTo<0 || m_nReqRangeTo>=nContLen) // open-end? set the end to file length. Also when request range would be too large
 						m_nReqRangeTo=nContLen-1;
 
+					// or simply don't care within that rage
+					bool bPermitPartialStart = (
+							fistate >= fileitem::FIST_DLGOTHEAD
+							&& fistate <= fileitem::FIST_COMPLETE
+							&& nGooddataSize >= ( m_nReqRangeFrom - acfg::maxredlsize));
+
 					/*
 					 * make sure that our client doesn't just hang here while the download thread is
 					 * fetching from 0 to start position for many minutes. If the resumed position
 					 * is beyond of what we already have, fall back to 200 (complete download).
 					 */
-					if(  fistate==fileitem::FIST_COMPLETE
+					if(fistate==fileitem::FIST_COMPLETE
 							// or can start sending within this range (positive range-from)
-							|| (
-									fistate >= fileitem::FIST_DLGOTHEAD
-									&& fistate <= fileitem::FIST_COMPLETE
-									&& nGooddataSize>=m_nReqRangeFrom
-								)
-							// don't care, found special hint from acngfs (kludge...)
-							|| m_pReqHead->h[header::ACNGFSMARK]
-
-						)
+							|| bPermitPartialStart	// don't care since found special hint from acngfs (kludge...)
+							|| m_pReqHead->h[header::ACNGFSMARK] )
 					{
 						// detect errors, out-of-range case
 						if(m_nReqRangeFrom>=nContLen || m_nReqRangeTo<m_nReqRangeFrom)
@@ -1128,13 +1154,13 @@ inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 	return 0;
 }
 
-fileitem::FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
+fileitem::FiStatus job::_SwitchToPtItem()
 {
 	// Changing to local pass-through file item
 	LOGSTART("job::_SwitchToPtItem");
 	// exception-safe sequence
-	m_pItem.ReplaceWithLocal(new tPassThroughFitem(m_sFileLoc));
-	return m_pItem->Setup(true);
+	m_pItem.RegisterFileitemLocalOnly(new tPassThroughFitem(m_sFileLoc));
+	return m_pItem.get()->Setup(true);
 }
 
 
@@ -1161,7 +1187,7 @@ void job::SetErrorResponse(const char * errorLine, const char *szLocation, const
 
 	erroritem *p = new erroritem("noid", errorLine, bodytext);
 	p->HeadRef().set(header::LOCATION, szLocation);
-	m_pItem.ReplaceWithLocal(p);;
+	m_pItem.RegisterFileitemLocalOnly(p);
 	//aclog::err(tSS() << "fileitem is now " << uintptr_t(m_pItem.get()));
 	m_state=STATE_SEND_MAIN_HEAD;
 }
